/// <reference types="vitest" />

import react from '@vitejs/plugin-react';
import { resolve } from 'path'
import { defineConfig } from 'vite';
import dts from 'vite-plugin-dts';
import path from 'path'

export default defineConfig({
  plugins: [
    react(),
    dts({
      insertTypesEntry: true,
    }),
  ],
  build: {
    lib: {
      entry: resolve(__dirname, './index.ts'),
      name: '@graphpolaris/shared',
      formats: ['es', 'umd'],
      fileName: (format) => `@graphpolaris-shared.${format}.js`,
    },
    rollupOptions: {
      external: ['react', 'react-dom'],
      output: {
        globals: {
          react: 'React',
          'react-dom': 'ReactDOM',
        },
      },
    },
  },
  resolve: {
    alias: {
      '@graphpolaris/shared/lib': path.resolve(__dirname, './lib')
    },
  },
  test: {
    setupFiles: ['./vitest.setup.ts'],
    environment: "happy-dom",
    deps: {
      // inline: ['vitest-canvas-mock'],
    },
    threads: false,
    environmentOptions: {
      jsdom: {
        resources: 'usable',
      },
    },
  }
});